package edu.sc.seis.gee.task;

import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;
import java.util.TimeZone;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.apache.log4j.Category;

import edu.iris.Fissures.AuditInfo;
import edu.iris.Fissures.Time;
import edu.iris.Fissures.IfNetwork.Channel;
import edu.iris.Fissures.IfNetwork.ChannelId;
import edu.iris.Fissures.IfNetwork.NetworkAccess;
import edu.iris.Fissures.IfNetwork.NetworkDCOperations;
import edu.iris.Fissures.IfSeismogramDC.DataCenterOperations;
import edu.iris.Fissures.IfSeismogramDC.RequestFilter;
import edu.iris.Fissures.model.MicroSecondDate;
import edu.iris.Fissures.model.TimeInterval;
import edu.iris.Fissures.model.UnitImpl;
import edu.iris.Fissures.network.NetworkIdUtil;
import edu.sc.seis.fissuresUtil.cache.BulletproofVestFactory;
import edu.sc.seis.fissuresUtil.cache.DataCenterRouter;
import edu.sc.seis.fissuresUtil.cache.HardCodeDataCenterRouter;
import edu.sc.seis.fissuresUtil.cache.InstrumentationLoader;
import edu.sc.seis.fissuresUtil.cache.NSNetworkDC;
import edu.sc.seis.fissuresUtil.cache.ProxySeismogramDC;
import edu.sc.seis.fissuresUtil.chooser.ChannelChooser;
import edu.sc.seis.fissuresUtil.chooser.ClockUtil;
import edu.sc.seis.fissuresUtil.chooser.DateChooser;
import edu.sc.seis.fissuresUtil.chooser.DateChooserOptions;
import edu.sc.seis.fissuresUtil.chooser.IntervalChooser;
import edu.sc.seis.fissuresUtil.chooser.IntervalChooserOptions;
import edu.sc.seis.fissuresUtil.database.DBDataCenter;
import edu.sc.seis.fissuresUtil.dataset.Organizer;
import edu.sc.seis.fissuresUtil.exceptionHandler.GlobalExceptionHandler;
import edu.sc.seis.fissuresUtil.namingService.FissuresNamingService;
import edu.sc.seis.fissuresUtil.xml.DCDataSetSeismogram;
import edu.sc.seis.fissuresUtil.xml.DataSetSeismogram;
import edu.sc.seis.gee.CommonAccess;
import edu.sc.seis.gee.NetworkGateKeeper;
import edu.sc.seis.gee.NoCorbaException;
import edu.sc.seis.gee.NoNetworkException;
import edu.sc.seis.gee.configurator.ConfigurationException;

/**
 * LoadSeismogram.java Created: Wed Jan 2 08:14:03 2002
 * 
 * @author <a href="mailto:">Srinivasa Telukutla </a>
 * @version
 */
public class LoadSeismogram implements GUITask {

    public LoadSeismogram() {
        dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));
        instLoader = new InstrumentationLoader();
        instLoader.start();
    }

    public void invoke() throws Exception {}

    public JComponent getMoreOptionsGUI() {
        return null;
    }

    public boolean hasMoreOptions() {
        return false;
    }

    public JComponent getGUI() {
        if(!guiInitialized) {
            guiInitialized = true;
            createGUI();
        } // end of if (! guiInitialized)
        return panel;
    }

    public void destroy() {}

    /***************************************************************************
     * This function is used to display the GUI used for the task LoadSeismogram
     * of the DataExplorer. The GUI contains a DateChooser(StartDate),
     * IntervalChooser, and PushButtons ok,Close and ChooseChannels.
     **************************************************************************/
    public void createGUI() {
        panel = new JPanel();
        IntervalChooserOptions[] options = new IntervalChooserOptions[6];
        options[0] = IntervalChooserOptions.SECOND;
        options[1] = IntervalChooserOptions.MINUTE;
        options[2] = IntervalChooserOptions.HOUR;
        options[3] = IntervalChooserOptions.DAY;
        options[4] = IntervalChooserOptions.MONTH;
        options[5] = IntervalChooserOptions.YEAR;
        final IntervalChooser intervalChooser = new IntervalChooser(options);
        intervalChooser.setSelectedValue(10);
        DateChooserOptions[] dateformat = new DateChooserOptions[6];
        dateformat[0] = DateChooserOptions.YEAR;
        dateformat[1] = DateChooserOptions.MONTH;
        dateformat[2] = DateChooserOptions.DAY;
        dateformat[3] = DateChooserOptions.HOUR;
        dateformat[4] = DateChooserOptions.MINUTES;
        dateformat[5] = DateChooserOptions.SECONDS;
        final DateChooser startDate = new DateChooser(dateformat);
        MicroSecondDate end = ClockUtil.now();
        MicroSecondDate begin = end.subtract(new TimeInterval(10,
                                                              UnitImpl.MINUTE));
        startDate.setDate(begin);
        JPanel endPanel = new JPanel();
        final JLabel endDateLabel = new JLabel("End Date");
        JLabel intervalLabel = new JLabel("Interval");
        JLabel startDateLabel = new JLabel("Start Date");
        JButton okButton = new JButton("OK");
        final GridBagLayout bagLayout = new GridBagLayout();
        final GridBagConstraints constraints = new GridBagConstraints();
        endPanel.setLayout(new FlowLayout());
        endPanel.add(okButton);
        panel.setLayout(bagLayout);
        constraints.weightx = 1.0;
        constraints.weighty = 1.0;
        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        ///adding start Date Label to the panel
        bagLayout.setConstraints(startDateLabel, constraints);
        panel.add(startDateLabel);
        constraints.weightx = 0;
        constraints.gridy = constraints.gridy + 1;
        bagLayout.setConstraints(startDate, constraints);
        panel.add(startDate);
        constraints.gridheight = 1;
        constraints.gridy = 2;
        constraints.gridx = 0;
        bagLayout.setConstraints(intervalLabel, constraints);
        panel.add(intervalLabel);
        constraints.gridx = 0;
        constraints.gridheight = 1;
        constraints.weightx = 1;
        constraints.gridy = constraints.gridy + 1;
        bagLayout.setConstraints(intervalChooser, constraints);
        panel.add(intervalChooser);
        constraints.gridx = 0;
        constraints.weighty = 1;
        constraints.gridy = constraints.gridy + 1;
        bagLayout.setConstraints(endDateLabel, constraints);
        panel.add(endDateLabel);
        constraints.gridy = constraints.gridy + 1;
        ;
        bagLayout.setConstraints(endPanel, constraints);
        panel.add(endPanel);
        okButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                try {
                    java.util.Date intervalDate = intervalChooser.addTo(startDate.getDate());
                    endDateLabel.setText("End Date:"
                            + dateFormat.format(intervalDate));
                    MicroSecondDate when = new MicroSecondDate(startDate.getDate());
                    Time startTime = when.getFissuresTime();
                    Time endTime = new MicroSecondDate(intervalDate).getFissuresTime();
                    Channel[] channels = channelChooserTask.getChannelChooser()
                            .getSelectedChannels(when);
                    ChannelId[] chanIds = new ChannelId[channels.length];
                    for(int i = 0; i < channels.length; i++) {
                        logger.debug(channels[i].get_id().station_code + "="
                                + channels[i].getSite().getStation().getName());
                        chanIds[i] = channels[i].get_id();
                    } // end of for ()
                    // make sure channel info is there before seis
                    populateLocalDataSet(channels);
                    loadAndStore(startTime, endTime, chanIds);
                } catch(Exception fe) {
                    GlobalExceptionHandler.handle("Exception while Getting the Seismograms",
                                                  fe);
                }
            }
        });
    }

    /**
     * Loads seismograms for the selected channels and times from the
     * seismogramDC server and stores them in a dataset using the current
     * DataSetOrganizer.
     */
    protected DataSetSeismogram[] loadAndStore(Time startTime,
                                               Time endTime,
                                               ChannelId[] channelId)
            throws ConfigurationException, NoNetworkException {
        DataSetSeismogram[] seis = new DataSetSeismogram[0];
        boolean tryAgain = true;
        while(tryAgain) {
            try {
                seis = retrieveSeismograms(startTime, endTime, channelId);
                populateLocalDataSet(seis);
                tryAgain = false;
            } catch(org.omg.CORBA.SystemException corbaE) {
                int n = JOptionPane.showConfirmDialog(panel,
                                                      "A problem has occured in connecting to the server."
                                                              + " Would you like to try again?",
                                                      "Network Error",
                                                      JOptionPane.OK_CANCEL_OPTION,
                                                      JOptionPane.ERROR_MESSAGE);
                if(n == JOptionPane.CANCEL_OPTION) {
                    tryAgain = false;
                    seis = new DataSetSeismogram[0];
                } else {
                    tryAgain = true;
                    dataCenter = null;
                } // end of else
            } // end of try-catch
        } // end of while (localSeismograms == null;)
        //populateLocalDataSet(seis);//populate the localdataset with the
        // seismograms retrieved.
        messageStr = new String("Retrieved " + seis.length + " Seismograms");
        JOptionPane.showMessageDialog(null,
                                      messageStr,
                                      "Seismogram Information",
                                      JOptionPane.INFORMATION_MESSAGE);
        return seis;
    }

    /**
     * This function retrieves Seismogram for the given startTime, endTime and
     * ChannelId
     */
    public DataSetSeismogram retrieveSeismogram(RequestFilter request)
            throws ConfigurationException, NoNetworkException {
        RequestFilter[] inRequest = new RequestFilter[] {request};
        RequestFilter[] outRequest = checkAvailableData(inRequest);
        if(outRequest.length != 0) {
            DataSetSeismogram seis = new DCDataSetSeismogram(request,
                                                             getDataCenter());
            ChannelChooser channelChooser = channelChooserTask.getChannelChooser();
            // only try to load instrumentation if network is alrady known
            if(channelChooser.isNetworkAccessKnown(inRequest[0].channel_id.network_id)) {
                instLoader.getInstrumentation(seis,
                                              channelChooser.getNetworkAccess(inRequest[0].channel_id.network_id));
            } else {
                logger.warn("networkAccess for "
                        + NetworkIdUtil.toString(outRequest[0].channel_id.network_id)
                        + " is not loaded, skipping instrumentation");
            }
            return seis;
        }
        return null;
    }

    public DataSetSeismogram retrieveSeismogram(Time startTime,
                                                Time endTime,
                                                ChannelId channelId)
            throws ConfigurationException, NoNetworkException {
        return retrieveSeismogram(new RequestFilter(channelId,
                                                    startTime,
                                                    endTime));
    }

    public DataSetSeismogram[] retrieveSeismograms(Time startTime,
                                                   Time endTime,
                                                   ChannelId[] channelId)
            throws ConfigurationException, NoNetworkException {
        LinkedList seismos = new LinkedList();
        for(int i = 0; i < channelId.length; i++) {
            DataSetSeismogram seis = retrieveSeismogram(new RequestFilter(channelId[i],
                                                                          startTime,
                                                                          endTime));
            if(seis != null) {
                seismos.add(seis);
            } // end of if ()
        } // end of for (int i=0; i<channelId.length; i++)
        DataSetSeismogram[] seismoArray = new DataSetSeismogram[0];
        return (DataSetSeismogram[])seismos.toArray(seismoArray);
    }

    public RequestFilter[] checkAvailableData(RequestFilter[] inFilters)
            throws ConfigurationException, NoNetworkException {
        if(!NetworkGateKeeper.corbaAccessAllowed()) { throw new NoCorbaException(); }
        //        return inFilters;
        RequestFilter[] outFilters;
        try {
            outFilters = getDataCenter().available_data(inFilters);
        } catch(org.omg.CORBA.SystemException e) {
            // try again???
            outFilters = getDataCenter().available_data(inFilters);
        }
        //logger.debug("Outfilters: "+outFilters.length+" for "+
        //               ChannelIdUtil.toStringNoDates(inFilters[0].channel_id));
        return outFilters;
    }

    /**
     * Adds seismograms vie the organizer. No channels are added.
     */
    public void populateLocalDataSet(DataSetSeismogram[] seismograms) {
        Organizer organizer = CommonAccess.getCommonAccess().getOrganizer();
        for(int i = 0; i < seismograms.length; i++) {
            if(seismograms[i] != null) {
                AuditInfo[] audit_info = {new AuditInfo(System.getProperty("user.name"),
                                                        "loaded seismogram "
                                                                + seismograms[i].getName())};
                organizer.addSeismogram(seismograms[i], audit_info);
            } else {
                logger.warn("Received a null seismogram");
            } // end of else
        } // end of for (int i=0; i<localSeismograms.length; i++)
    }

    public void populateLocalDataSet(Channel[] channels) {
        Organizer organizer = CommonAccess.getCommonAccess().getOrganizer();
        AuditInfo[] audit = new AuditInfo[1];
        audit[0] = new AuditInfo(System.getProperty("user.name"),
                                 "loaded channel.");
        for(int counter = 0; counter < channels.length; counter++) {
            organizer.addChannel(channels[counter], audit);
        }
    }

    public void configure(java.util.Map params) throws ConfigurationException {
        configParams = params;
        String chanChooserId = (String)params.get("channelChooserTask");
        TaskAction chanChooserAction = CommonAccess.getCommonAccess()
                .getTaskAction(chanChooserId);
        channelChooserTask = (ChannelChooserTask)chanChooserAction.getTask();
    }

    void initializeDCRouter() throws ConfigurationException {
        if(!NetworkGateKeeper.corbaAccessAllowed()) { return; }
        CommonAccess commonAccess = CommonAccess.getCommonAccess();
        FissuresNamingService fissuresNamingService = commonAccess.getFissuresNamingService();
        dataCenterRouter = new HardCodeDataCenterRouter(fissuresNamingService);
        try {
            dataCenter = DBDataCenter.getDataCenter(commonAccess.getCacheDirectory(),
                                                    "GEE_database",
                                                    dataCenterRouter);
        } catch(SQLException e) {
            throw new ConfigurationException("Database problem.", e);
        } // end of try-catch
    }

    void addDefaults(DataCenterRouter router) throws ConfigurationException {
        Set keySet = configParams.keySet();
        Iterator iter = keySet.iterator();
        while(iter.hasNext()) {
            String key = (String)iter.next();
            if(key.startsWith(SERVER_DNS)) {
                String dnsValue = (String)configParams.get(key);
                String nameValue = (String)configParams.get(SERVER_NAME
                        + key.substring(9));
                logger.debug(key + " " + key.substring(9) + " " + nameValue);
                if(dnsValue == null && dnsValue.length() == 0
                        && nameValue == null && nameValue.length() == 0) { throw new ConfigurationException("Bad configuration for "
                        + key + " and " + (String)configParams.get(key)); } // end
                // of
                // if ()
                ProxySeismogramDC dc = getDataCenter(dnsValue, nameValue);
                dataCenterRouter.addDataCenter(dc);
            }
        }
    }

    void addNetworkToRouter(DataCenterRouter router, NetworkAccess net)
            throws ConfigurationException, NoNetworkException {
        NetworkDCOperations[] netDCs = channelChooserTask.getChannelChooser()
                .getNetworkDCs();
        Map netDCToNetMap = channelChooserTask.getChannelChooser()
                .getNetDCToNetMap();
        logger.debug("netDCToNetMap size is " + netDCToNetMap.size()
                + "  netDCs length =" + netDCs.length);
        Set keySet = configParams.keySet();
        Iterator iter = keySet.iterator();
        while(iter.hasNext()) {
            String key = (String)iter.next();
            if(key.startsWith(SERVER_DNS)) {
                String dnsValue = (String)configParams.get(key);
                String nameValue = (String)configParams.get(SERVER_NAME
                        + key.substring(9));
                logger.debug(key + " " + key.substring(9) + " " + nameValue);
                if(dnsValue == null && dnsValue.length() == 0
                        && nameValue == null && nameValue.length() == 0) { throw new ConfigurationException("Bad configuration for "
                        + key + " and " + (String)configParams.get(key)); } // end
                // of
                // if ()
                try {
                    boolean found = false;
                    NSNetworkDC nsnetDC = null;
                    for(int i = 0; found == false && i < netDCs.length; i++) {
                        nsnetDC = (NSNetworkDC)netDCs[i];
                        NetworkAccess[] nets = (NetworkAccess[])netDCToNetMap.get(netDCs[i]);
                        if(nets != null) {
                            for(int j = 0; j < nets.length; j++) {
                                logger.debug("nets for dc are " + j + " "
                                        + nets[j].get_attributes().get_code());
                                if(nets[j] == net) {
                                    // found it
                                    found = true;
                                    break;
                                }
                            }
                        }
                    }
                    logger.debug("try to match serverDNS "
                            + nsnetDC.getServerDNS() + " " + dnsValue);
                    logger.debug("nsnetDC=" + nsnetDC + " "
                            + netDCToNetMap.keySet().size());
                    if(nsnetDC.getServerDNS().equals(dnsValue)) {
                        // assume seis servers from the same DNS as
                        // network servers have the same networks
                        ProxySeismogramDC dc = getDataCenter(dnsValue,
                                                             nameValue);
                        dataCenterRouter.addDataCenter(net, dc);
                    }
                } catch(Exception e) {
                    logger.error("Exception loading seismogram datacenters", e);
                    throw new ConfigurationException("Unable to resolve"
                            + dnsValue + " " + nameValue + " FVer="
                            + edu.iris.Fissures.VERSION.value, e);
                }
            }
        }
    }

    public ProxySeismogramDC getDataCenter(String dns, String name)
            throws ConfigurationException {
        String key = dns + "/" + name;
        if(nameToDataCenterMap.containsKey(key)) {
            return (ProxySeismogramDC)nameToDataCenterMap.get(key);
        } else {
            CommonAccess commonAccess = CommonAccess.getCommonAccess();
            FissuresNamingService fissuresNamingService = commonAccess.getFissuresNamingService();
            ProxySeismogramDC seisDC = BulletproofVestFactory.vestSeismogramDC(dns,
                                                                               name,
                                                                               fissuresNamingService);
            nameToDataCenterMap.put(key, seisDC);
            return seisDC;
        }
    }

    public DataCenterOperations getDataCenter() throws ConfigurationException {
        if(dataCenter == null) {
            initializeDCRouter();
        } // end of if (dataCenter == null)
        return dataCenter;
    }

    public DataCenterRouter getDataCenterRouter() throws ConfigurationException {
        if(dataCenterRouter == null) {
            initializeDCRouter();
        } // end of if (dataCenter == null)
        return dataCenterRouter;
    }

    java.text.SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy MMM dd HH:mm:ss z");

    protected boolean guiInitialized = false;

    protected JPanel panel = null;

    protected DataCenterRouter dataCenterRouter;

    protected DataCenterOperations dataCenter;

    protected HashMap nameToDataCenterMap = new HashMap();

    private Map configParams;

    private ChannelChooserTask channelChooserTask;

    private String messageStr = new String();

    private InstrumentationLoader instLoader;

    static Category logger = Category.getInstance(LoadSeismogram.class.getName());
}// LoadSeismogram
